#!/bin/bash

set -e

# Load conf and consts
. /opt/txr/txr.conf

function clear_tcr_gre {
  for dev in `ip tunnel show | grep tcr_gre | cut -f 1 -d ":"`; do
    ip tunnel del $dev
  done
  while [ `ip rule show | grep -c "$PROTO:"` -ne 0 ]; do ip rule del pref $PROTO; done
}

function cleanup {
  trap - EXIT INT TERM
  ip tunnel del $GRE_NEXT_DEV 2>/dev/null ||:
  ip tunnel del $GRE_OTHER_DEV 2>/dev/null ||:
  sh -c '/opt/txr/iptables/iptables-restore < /opt/txr/txr_iptables_default_policy'
  if [ -z $STATIC_ROUTE_NETWORK ]; then
    service zebra stop
    service bgpd stop
  else
    ip route flush proto $PROTO
  fi
  /sbin/rmmod /opt/txr/iptables/xt_auth.ko 2>/dev/null ||:
  clear_tcr_gre
  exit 0
}

trap cleanup EXIT INT TERM

function initialize {
  DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
  export LD_LIBRARY_PATH=/opt/txr/

  modprobe ip_gre
  sysctl -w net.ipv4.ip_forward=1
  sysctl -w net.ipv4.conf.all.rp_filter=0
  sysctl -w net.ipv4.conf.default.rp_filter=0
  
  /opt/txr/iptables/iptables-restore < /opt/txr/txr_iptables_default_policy
  /sbin/rmmod /opt/txr/iptables/xt_auth.ko 2>/dev/null ||:
  VER=`uname -r | awk -F '.' '{print $1\".\"$2\".\"$3}'`
  /sbin/insmod /opt/txr/iptables/xt_auth_$VER.ko

  clear_tcr_gre
  
  if [ -z $STATIC_ROUTE_NETWORK ]; then
    service zebra stop
    service bgpd stop
  else
    ip route flush proto $PROTO
  fi
}

function create_tunnels {
  ip tunnel del $GRE_OTHER_DEV 2>/dev/null ||:
  ip tunnel del $GRE_NEXT_DEV 2>/dev/null ||:  

  ip tunnel add $GRE_NEXT_DEV mode gre local `ip route get $GRE_NEXT_IP | head -1 | awk '{ print $7}'` remote $GRE_NEXT_IP ttl 255
  iptables -t mangle -I PREROUTING -i $GRE_NEXT_DEV -j CONNMARK --restore-mark
  if [ ! -z $GRE_OTHER_IP ]; then
    ip tunnel add $GRE_OTHER_DEV mode gre local `ip route get $GRE_OTHER_IP | head -1 | awk '{ print $7}'` remote $GRE_OTHER_IP ttl 255
    iptables -t mangle -I PREROUTING -i $GRE_OTHER_DEV -j CONNMARK --restore-mark
  fi
}

function setup_tunnels {
  # set the correct address according to BGP scripts
  if [ "$MODE" = "active" ]; then
    GRE_OTHER_ADDR=10.0.0.9/30
    GRE_NEXT_ADDR=10.0.0.1/30
  elif [ "$MODE" = "passive" ]; then
    GRE_OTHER_ADDR=10.0.0.10/30
    GRE_NEXT_ADDR=10.0.0.5/30
  else 
    echo "Unknown role: $MODE" >&2
    exit 1
  fi
  
  if [ ! -z $GRE_OTHER_IP ]; then
    ip addr add dev $GRE_OTHER_DEV $GRE_OTHER_ADDR
    ip link set $GRE_OTHER_DEV up
  fi
  ip addr add dev $GRE_NEXT_DEV $GRE_NEXT_ADDR
  ip link set $GRE_NEXT_DEV up
}

function run {
  cp /opt/txr/bgpd.conf.$MODE /etc/quagga/bgpd.conf
  cp /opt/txr/zebra.conf.$MODE /etc/quagga/zebra.conf

  /opt/txr/iptables/iptables -A INPUT -p icmp --icmp-type echo-request -j ACCEPT
  /opt/txr/iptables/iptables -A INPUT -p tcp --dport 179 -j ACCEPT
  /opt/txr/iptables/iptables -A INPUT -p tcp --dport $PROBE_PORT -m state --state NEW -m tcp --tcp-flags SYN,RST,ACK SYN -m auth --key $KEY --grace 900 -j ACCEPT

  if [ -z $STATIC_ROUTE_NETWORK ]; then
    service zebra start
    service bgpd start
  else
    ip route add $STATIC_ROUTE_NETWORK dev $GRE_NEXT_DEV proto $PROTO
  fi

  python $DGRE -p $PROBE_PORT &
  wait
}

initialize
create_tunnels
setup_tunnels
run
